# Copyright 2013 UnitedStack Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import shlex

from neutronclient.common import exceptions as n_exc
from oslo_log import log as logging
from oslo_utils import strutils
from oslo_utils import uuidutils
import pecan
import six

from zun.api.controllers import base
from zun.api.controllers import link
from zun.api.controllers.v1 import collection
from zun.api.controllers.v1.schemas import containers as schema
from zun.api.controllers.v1.views import actions_view
from zun.api.controllers.v1.views import containers_view as view
from zun.api import utils as api_utils
from zun.api import validation
from zun.common import consts
from zun.common import context as zun_context
from zun.common import exception
from zun.common.i18n import _
from zun.common import name_generator
from zun.common.policies import container as policies
from zun.common import policy
from zun.common import quota
from zun.common import utils
import zun.conf
from zun.network import model as network_model
from zun.network import neutron
from zun import objects
from zun.pci import request as pci_request
from zun.volume import cinder_api as cinder

CONF = zun.conf.CONF
LOG = logging.getLogger(__name__)
QUOTAS = quota.QUOTAS


def check_policy_on_container(container, action):
    context = pecan.request.context
    policy.enforce(context, action, container, action=action)


class ContainerCollection(collection.Collection):
    """API representation of a collection of containers."""

    fields = {
        'containers',
        'next'
    }

    """A list containing containers objects"""

    def __init__(self, **kwargs):
        super(ContainerCollection, self).__init__(**kwargs)
        self._type = 'containers'

    @staticmethod
    def convert_with_links(rpc_containers, limit, url=None,
                           expand=False, **kwargs):
        context = pecan.request.context
        collection = ContainerCollection()
        collection.containers = \
            [view.format_container(context, url, p)
             for p in rpc_containers]
        collection.next = collection.get_next(limit, url=url, **kwargs)
        return collection


class ContainersActionsController(base.Controller):
    """Controller for Container Actions."""

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_all(self, container_ident, **kwargs):
        """Retrieve a list of container actions."""
        context = pecan.request.context
        policy.enforce(context, "container:actions",
                       action="container:actions")
        container = utils.get_container(container_ident)
        actions_raw = objects.ContainerAction.get_by_container_uuid(
            context, container.uuid)
        actions = [actions_view.format_action(a) for a in actions_raw]

        return {"containerActions": actions}

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_one(self, container_ident, request_ident, **kwargs):
        """Retrieve information about the action."""

        context = pecan.request.context
        policy.enforce(context, "container:actions",
                       action="container:actions")
        container = utils.get_container(container_ident)
        action = objects.ContainerAction.get_by_request_id(
            context, container.uuid, request_ident)

        if action is None:
            raise exception.ResourceNotFound(name="Action", id=request_ident)

        action_id = action.id
        action = actions_view.format_action(action)
        show_traceback = False
        if policy.enforce(context, "container:action:events",
                          do_raise=False, action="container:action:events"):
            show_traceback = True

        events_raw = objects.ContainerActionEvent.get_by_action(context,
                                                                action_id)
        action['events'] = [actions_view.format_event(evt, show_traceback)
                            for evt in events_raw]
        return action


class ContainersController(base.Controller):
    """Controller for Containers."""

    _custom_actions = {
        'start': ['POST'],
        'stop': ['POST'],
        'reboot': ['POST'],
        'rebuild': ['POST'],
        'pause': ['POST'],
        'unpause': ['POST'],
        'logs': ['GET'],
        'execute': ['POST'],
        'execute_resize': ['POST'],
        'kill': ['POST'],
        'rename': ['POST'],
        'attach': ['GET'],
        'resize': ['POST'],
        'resize_container': ['POST'],
        'top': ['GET'],
        'get_archive': ['GET'],
        'put_archive': ['POST'],
        'stats': ['GET'],
        'commit': ['POST'],
        'add_security_group': ['POST'],
        'network_detach': ['POST'],
        'network_attach': ['POST'],
        'network_list': ['GET'],
        'remove_security_group': ['POST']
    }

    container_actions = ContainersActionsController()

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_all(self, **kwargs):
        """Retrieve a list of containers.

        """
        context = pecan.request.context
        policy.enforce(context, "container:get_all",
                       action="container:get_all")
        return self._get_containers_collection(**kwargs)

    def _get_containers_collection(self, **kwargs):
        context = pecan.request.context
        if utils.is_all_projects(kwargs):
            policy.enforce(context, "container:get_all_all_projects",
                           action="container:get_all_all_projects")
            context.all_projects = True
        kwargs.pop('all_projects', None)
        limit = api_utils.validate_limit(kwargs.pop('limit', None))
        sort_dir = api_utils.validate_sort_dir(kwargs.pop('sort_dir', 'asc'))
        sort_key = kwargs.pop('sort_key', 'id')
        resource_url = kwargs.pop('resource_url', None)
        expand = kwargs.pop('expand', None)

        container_allowed_filters = ['name', 'image', 'project_id', 'user_id',
                                     'memory', 'host', 'task_state', 'status',
                                     'auto_remove']
        filters = {}
        for filter_key in container_allowed_filters:
            if filter_key in kwargs:
                policy_action = policies.CONTAINER % ('get_one:' + filter_key)
                context.can(policy_action, might_not_exist=True)
                filter_value = kwargs.pop(filter_key)
                filters[filter_key] = filter_value
        marker_obj = None
        marker = kwargs.pop('marker', None)
        if marker:
            marker_obj = objects.Container.get_by_uuid(context,
                                                       marker)
        if kwargs:
            unknown_params = [str(k) for k in kwargs]
            msg = _("Unknown parameters: %s") % ", ".join(unknown_params)
            raise exception.InvalidValue(msg)

        containers = objects.Container.list(context,
                                            limit,
                                            marker_obj,
                                            sort_key,
                                            sort_dir,
                                            filters=filters)
        return ContainerCollection.convert_with_links(containers, limit,
                                                      url=resource_url,
                                                      expand=expand,
                                                      sort_key=sort_key,
                                                      sort_dir=sort_dir)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_one(self, container_ident, **kwargs):
        """Retrieve information about the given container.

        :param container_ident: UUID or name of a container.
        """
        context = pecan.request.context
        if utils.is_all_projects(kwargs):
            policy.enforce(context, "container:get_one_all_projects",
                           action="container:get_one_all_projects")
            context.all_projects = True
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:get_one")
        if container.host:
            compute_api = pecan.request.compute_api
            try:
                container = compute_api.container_show(context, container)
            except exception.ContainerHostNotUp as e:
                LOG.error("Failed to get container details: %s", str(e))

        return view.format_container(context, pecan.request.host_url,
                                     container)

    def _generate_name_for_container(self):
        """Generate a random name like: zeta-22-container."""
        name_gen = name_generator.NameGenerator()
        name = name_gen.generate()
        return name + '-container'

    @base.Controller.api_version("1.1", "1.19")
    @pecan.expose('json')
    @api_utils.enforce_content_types(['application/json'])
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_create)
    @validation.validated(schema.legacy_container_create)
    def post(self, run=False, **container_dict):
        # NOTE(hongbin): We convert the representation of 'command' from
        # string to list. For example:
        # '"nginx" "-g" "daemon off;"' -> ["nginx", "-g", "daemon off;"]
        command = container_dict.pop('command', None)
        if command is not None:
            if isinstance(command, six.string_types):
                command = shlex.split(command)
            container_dict['command'] = command

        return self._do_post(run, **container_dict)

    @base.Controller.api_version("1.20", "1.35")  # noqa
    @pecan.expose('json')
    @api_utils.enforce_content_types(['application/json'])
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_create)
    @validation.validated(schema.container_create)
    def post(self, run=False, **container_dict):
        interactive = container_dict.get('interactive')
        if interactive is not None:
            container_dict['tty'] = interactive
        return self._do_post(run, **container_dict)

    @base.Controller.api_version("1.36")  # noqa
    @pecan.expose('json')
    @api_utils.enforce_content_types(['application/json'])
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_create)
    @validation.validated(schema.container_create)
    def post(self, run=False, **container_dict):
        return self._do_post(run, **container_dict)

    def _do_post(self, run=False, **container_dict):
        """Create or run a new container.

        :param run: if true, starts the container
        :param container_dict: a container within the request body.
        """
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        policy.enforce(context, "container:create",
                       action="container:create")

        if container_dict.get('security_groups'):
            # remove duplicate security_groups from list
            container_dict['security_groups'] = list(set(
                container_dict.get('security_groups')))
            for index, sg in enumerate(container_dict['security_groups']):
                security_group_id = self._check_security_group(context,
                                                               {'name': sg})
                container_dict['security_groups'][index] = security_group_id

        try:
            run = strutils.bool_from_string(run, strict=True)
            container_dict['interactive'] = strutils.bool_from_string(
                container_dict.get('interactive', False), strict=True)
        except ValueError:
            bools = ', '.join(strutils.TRUE_STRINGS + strutils.FALSE_STRINGS)
            raise exception.InvalidValue(_('Valid run or interactive '
                                           'values are: %s') % bools)

        # Check container quotas
        self._check_container_quotas(context, container_dict)

        auto_remove = container_dict.pop('auto_remove', None)
        if auto_remove is not None:
            api_utils.version_check('auto_remove', '1.3')
            try:
                container_dict['auto_remove'] = strutils.bool_from_string(
                    auto_remove, strict=True)
            except ValueError:
                bools = ', '.join(strutils.TRUE_STRINGS +
                                  strutils.FALSE_STRINGS)
                raise exception.InvalidValue(_('Valid auto_remove '
                                               'values are: %s') % bools)

        runtime = container_dict.pop('runtime', None)
        if runtime is not None:
            api_utils.version_check('runtime', '1.5')
            policy.enforce(context, "container:create:runtime",
                           action="container:create:runtime")
            container_dict['runtime'] = runtime

        hostname = container_dict.pop('hostname', None)
        if hostname is not None:
            api_utils.version_check('hostname', '1.9')
            container_dict['hostname'] = hostname

        nets = container_dict.get('nets', [])
        requested_networks = utils.build_requested_networks(context, nets)
        pci_req = self._create_pci_requests_for_sriov_ports(context,
                                                            requested_networks)

        healthcheck = container_dict.pop('healthcheck', {})
        if healthcheck:
            api_utils.version_check('healthcheck', '1.22')
            healthcheck['test'] = healthcheck.pop('cmd', '')
            container_dict['healthcheck'] = healthcheck

        mounts = container_dict.pop('mounts', [])
        if mounts:
            api_utils.version_check('mounts', '1.11')

        cpu_policy = container_dict.pop('cpu_policy', None)
        container_dict['cpu_policy'] = cpu_policy

        privileged = container_dict.pop('privileged', None)
        if privileged is not None:
            api_utils.version_check('privileged', '1.21')
            policy.enforce(context, "container:create:privileged",
                           action="container:create:privileged")
            try:
                container_dict['privileged'] = strutils.bool_from_string(
                    privileged, strict=True)
            except ValueError:
                bools = ', '.join(strutils.TRUE_STRINGS +
                                  strutils.FALSE_STRINGS)
                raise exception.InvalidValue(_('Valid privileged values '
                                               'are: %s') % bools)

        # Valiadtion accepts 'None' so need to convert it to None
        container_dict['image_driver'] = api_utils.string_or_none(
            container_dict.get('image_driver'))
        if not container_dict['image_driver']:
            container_dict['image_driver'] = CONF.default_image_driver
        if container_dict.get('image_pull_policy'):
            policy.enforce(context, "container:create:image_pull_policy",
                           action="container:create:image_pull_policy")

        container_dict['project_id'] = context.project_id
        container_dict['user_id'] = context.user_id
        name = container_dict.get('name') or \
            self._generate_name_for_container()
        container_dict['name'] = name
        self._set_default_resource_limit(container_dict)
        if container_dict.get('restart_policy'):
            utils.check_for_restart_policy(container_dict)

        exposed_ports = container_dict.pop('exposed_ports', None)
        if exposed_ports is not None:
            api_utils.version_check('exposed_ports', '1.24')
            exposed_ports = utils.build_exposed_ports(exposed_ports)
            container_dict['exposed_ports'] = exposed_ports

        registry = container_dict.pop('registry', None)
        if registry:
            api_utils.version_check('registry', '1.31')
            registry = utils.get_registry(registry)
            container_dict['registry_id'] = registry.id

        container_dict['status'] = consts.CREATING
        extra_spec = {}
        extra_spec['hints'] = container_dict.get('hints', None)
        extra_spec['pci_requests'] = pci_req
        extra_spec['availability_zone'] = container_dict.get(
            'availability_zone')
        new_container = objects.Container(context, **container_dict)
        new_container.create(context)

        kwargs = {}
        kwargs['extra_spec'] = extra_spec
        kwargs['requested_networks'] = requested_networks
        kwargs['requested_volumes'] = (
            self._build_requested_volumes(context, new_container, mounts))
        if pci_req.requests:
            kwargs['pci_requests'] = pci_req
        kwargs['run'] = run
        compute_api.container_create(context, new_container, **kwargs)
        # Set the HTTP Location Header
        pecan.response.location = link.build_url('containers',
                                                 new_container.uuid)
        pecan.response.status = 202
        return view.format_container(context, pecan.request.host_url,
                                     new_container)

    def _check_container_quotas(self, context, container_delta_dict,
                                update_container=False):
        deltas = {
            'containers': 0 if update_container else 1,
            'cpu': container_delta_dict.get('cpu', 0),
            'memory': container_delta_dict.get('memory', 0),
            'disk': container_delta_dict.get('disk', 0)
        }

        def _check_deltas(context, deltas):
            """Check usage deltas against quota limits.

            This does QUOTAS.count() followed by a QUOTAS.limit_check()
            using the provided deltas.

            :param context: The request context, for access check.
            :param deltas: A dict of {resource_name: delta, ...} to check
                           against the quota limits.
            """
            check_kwargs = {}
            count_as_dict = {}
            project_id = context.project_id
            for res_name, res_delta in deltas.items():
                # TODO(kiennt): Apply count_as_dict method, query count usage
                #               once rather than count each resource.
                count_as_dict[res_name] = QUOTAS.count(context, res_name,
                                                       project_id)
                total = None
                try:
                    if isinstance(count_as_dict[res_name], six.integer_types):
                        total = count_as_dict[res_name] + int(res_delta)
                    else:
                        total = float(count_as_dict[res_name]) + \
                            float(res_delta)
                except TypeError as e:
                    raise e
                check_kwargs[res_name] = total
            QUOTAS.limit_check(context, project_id, **check_kwargs)

        _check_deltas(context, deltas)

    def _set_default_resource_limit(self, container_dict):
        # NOTE(kiennt): Default disk size will be set later.
        container_dict['disk'] = container_dict.get('disk')
        container_dict['memory'] = container_dict.get(
            'memory', CONF.default_memory)
        container_dict['memory'] = str(container_dict['memory'])
        container_dict['cpu'] = container_dict.get(
            'cpu', CONF.default_cpu)

    def _create_pci_requests_for_sriov_ports(self, context,
                                             requested_networks):
        pci_requests = objects.ContainerPCIRequests(requests=[])
        if not requested_networks:
            return pci_requests

        neutron_api = neutron.NeutronAPI(context)
        for request_net in requested_networks:
            phynet_name = None
            vnic_type = network_model.VNIC_TYPE_NORMAL

            if request_net.get('port'):
                vnic_type, phynet_name = self._get_port_vnic_info(
                    context, neutron_api, request_net['port'])
            pci_request_id = None
            if vnic_type in network_model.VNIC_TYPES_SRIOV:
                spec = {pci_request.PCI_NET_TAG: phynet_name}
                dev_type = pci_request.DEVICE_TYPE_FOR_VNIC_TYPE.get(vnic_type)
                if dev_type:
                    spec[pci_request.PCI_DEVICE_TYPE_TAG] = dev_type
                request = objects.ContainerPCIRequest(
                    count=1,
                    spec=[spec],
                    request_id=uuidutils.generate_uuid())
                pci_requests.requests.append(request)
                pci_request_id = request.request_id
            request_net['pci_request_id'] = pci_request_id
        return pci_requests

    def _get_port_vnic_info(self, context, neutron, port_id):
        """Retrieve port vnic info

        Invoked with a valid port_id.
        Return vnic type and the attached physical network name.
        """
        phynet_name = None
        port = self._show_port(context, port_id, neutron_client=neutron,
                               fields=['binding:vnic_type', 'network_id'])
        vnic_type = port.get('binding:vnic_type',
                             network_model.VNIC_TYPE_NORMAL)
        if vnic_type in network_model.VNIC_TYPES_SRIOV:
            net_id = port['network_id']
            phynet_name = self._get_phynet_info(context, net_id)
        return vnic_type, phynet_name

    def _show_port(self, context, port_id, neutron_client=None, fields=None):
        """Return the port for the client given the port id.

        :param context: Request context.
        :param port_id: The id of port to be queried.
        :param neutron_client: A neutron client.
        :param fields: The condition fields to query port data.
        :returns: A dict of port data.
                  e.g. {'port_id': 'abcd', 'fixed_ip_address': '1.2.3.4'}
        """
        if not neutron_client:
            neutron_client = neutron.NeutronAPI(context)
        if fields:
            result = neutron_client.show_port(port_id, fields=fields)
        else:
            result = neutron_client.show_port(port_id)
        return result.get('port')

    def _get_phynet_info(self, context, net_id):
        # NOTE(hongbin): Use admin context here because non-admin users are
        # unable to retrieve provider:* attributes.
        admin_context = zun_context.get_admin_context()
        neutron_api = neutron.NeutronAPI(admin_context)
        network = neutron_api.show_network(
            net_id, fields='provider:physical_network')
        net = network.get('network')
        phynet_name = net.get('provider:physical_network')
        return phynet_name

    def _build_requested_volumes(self, context, container, mounts):
        cinder_api = cinder.CinderAPI(context)
        requested_volumes = {container.uuid: []}
        for mount in mounts:
            volume_dict = {
                'cinder_volume_id': None,
                'container_path': None,
                'auto_remove': False,
                'contents': None,
                'user_id': context.user_id,
                'project_id': context.project_id,
            }
            volume_type = mount.get('type', 'volume')
            if volume_type == 'volume':
                if mount.get('source'):
                    volume = cinder_api.search_volume(mount['source'])
                    cinder_api.ensure_volume_usable(volume)
                    volume_dict['cinder_volume_id'] = volume.id
                    volume_dict['container_path'] = mount['destination']
                    volume_dict['volume_provider'] = 'cinder'
                elif mount.get('size'):
                    volume = cinder_api.create_volume(mount['size'])
                    cinder_api.ensure_volume_usable(volume)
                    volume_dict['cinder_volume_id'] = volume.id
                    volume_dict['container_path'] = mount['destination']
                    volume_dict['volume_provider'] = 'cinder'
                    volume_dict['auto_remove'] = True
            elif volume_type == 'bind':
                volume_dict['contents'] = mount.pop('source', '')
                volume_dict['container_path'] = mount['destination']
                volume_dict['volume_provider'] = 'local'

            volmapp = objects.VolumeMapping(context, **volume_dict)
            requested_volumes[container.uuid].append(volmapp)

        return requested_volumes

    def _check_security_group(self, context, security_group):
        neutron_api = neutron.NeutronAPI(context)
        try:
            return neutron_api.find_resourceid_by_name_or_id(
                'security_group', security_group['name'], context.project_id)
        except n_exc.NeutronClientNoUniqueMatch as e:
            msg = _("Multiple security group matches found for name "
                    "%(name)s, use an ID to be more specific.") % {
                'name': security_group['name']}
            raise exception.Conflict(msg)
        except n_exc.NeutronClientException as e:
            if e.status_code == 404:
                msg = _("Security group %(name)s not found.") % {
                    'name': security_group['name']}
                raise exception.InvalidValue(msg)
            else:
                raise

    @base.Controller.api_version("1.1", "1.14")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.add_security_group)
    def add_security_group(self, container_ident, **security_group):
        """Add security group to an existing container.

        :param container_ident: UUID or Name of a container.
        :param security_group: security_group to be added to container.
        """

        container = utils.get_container(container_ident)
        check_policy_on_container(
            container.as_dict(), "container:add_security_group")
        utils.validate_container_state(container, 'add_security_group')

        # check if security group already presnt in container
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        security_group_id = self._check_security_group(context, security_group)
        if security_group_id in container.security_groups:
            msg = _("Security group %(id)s has been added to container.") % {
                'id': security_group_id}
            raise exception.InvalidValue(msg)
        compute_api.add_security_group(context, container,
                                       security_group_id)
        pecan.response.status = 202

    @base.Controller.api_version("1.1", "1.14")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.remove_security_group)
    def remove_security_group(self, container_ident, **security_group):
        """Remove security group from an existing container.

        :param container_ident: UUID or Name of a container.
        :param security_group: security_group to be removed from container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(
            container.as_dict(), "container:remove_security_group")
        utils.validate_container_state(container, 'remove_security_group')

        context = pecan.request.context
        compute_api = pecan.request.compute_api
        security_group_id = self._check_security_group(context, security_group)
        if security_group_id not in container.security_groups:
            msg = _("Security group %(id)s was not added to container.") % {
                'id': security_group_id}
            raise exception.InvalidValue(msg)
        compute_api.remove_security_group(context, container,
                                          security_group_id)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.container_update)
    def patch(self, container_ident, **patch):
        """Update an existing container.

        :param container_ident: UUID or name of a container.
        :param patch: a json PATCH document to apply to this container.
        """
        container = utils.get_container(container_ident)
        context = pecan.request.context
        container_deltas = {}
        check_policy_on_container(container.as_dict(), "container:update")
        utils.validate_container_state(container, 'update')
        if 'memory' in patch:
            container_deltas['memory'] = \
                int(patch['memory']) - int(container.memory)
            patch['memory'] = str(patch['memory'])
        if 'cpu' in patch:
            patch['cpu'] = float(patch['cpu'])
            container_deltas['cpu'] = patch['cpu'] - container.cpu
        if 'name' in patch:
            patch['name'] = str(patch['name'])

        if 'memory' not in patch and 'cpu' not in patch:
            for field, patch_val in patch.items():
                if getattr(container, field) != patch_val:
                    setattr(container, field, patch_val)
            container.save(context)
        else:
            # Check container quotas
            self._check_container_quotas(context, container_deltas,
                                         update_container=True)
            compute_api = pecan.request.compute_api
            container = compute_api.container_update(context, container, patch)
        return view.format_container(context, pecan.request.host_url,
                                     container)

    @base.Controller.api_version("1.1", "1.13")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_rename)
    def rename(self, container_ident, name):
        """Rename an existing container.

        :param container_ident: UUID or Name of a container.
        :param name: a new name for this container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:rename")
        if container.name == name:
            raise exception.Conflict('The new name for the container is the '
                                     'same as the old name.')
        container.name = name
        context = pecan.request.context
        container.save(context)
        return view.format_container(context, pecan.request.host_url,
                                     container)

    @base.Controller.api_version("1.19")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.container_update)
    def resize_container(self, container_ident, **kwargs):
        """Resize an existing container.

        :param container_ident: UUID or name of a container.
        :param kwargs: cpu/memory to be updated.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(),
                                  "container:resize_container")
        utils.validate_container_state(container, 'resize_container')
        if 'memory' in kwargs:
            kwargs['memory'] = str(kwargs['memory'])
        if 'cpu' in kwargs:
            kwargs['cpu'] = float(kwargs['cpu'])
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.resize_container(context, container, kwargs)
        pecan.response.status = 202
        return view.format_container(context, pecan.request.host_url,
                                     container)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_delete)
    def delete(self, container_ident, force=False, **kwargs):
        """Delete a container.

        :param container_ident: UUID or Name of a container.
        :param force: If True, allow to force delete the container.
        """
        context = pecan.request.context
        if utils.is_all_projects(kwargs):
            policy.enforce(context, "container:delete_all_projects",
                           action="container:delete_all_projects")
            context.all_projects = True
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:delete")
        stop = kwargs.pop('stop', False)
        try:
            force = strutils.bool_from_string(force, strict=True)
            stop = strutils.bool_from_string(stop, strict=True)
        except ValueError:
            bools = ', '.join(strutils.TRUE_STRINGS + strutils.FALSE_STRINGS)
            raise exception.InvalidValue(_('Valid force or stop values '
                                           'are: %s') % bools)
        compute_api = pecan.request.compute_api
        if not force and not stop:
            utils.validate_container_state(container, 'delete')
        elif force and not stop:
            api_utils.version_check('force', '1.7')
            policy.enforce(context, "container:delete_force",
                           action="container:delete_force")
            utils.validate_container_state(container, 'delete_force')
        elif stop:
            api_utils.version_check('stop', '1.12')
            utils.validate_container_state(container,
                                           'delete_after_stop')
            if container.status == consts.RUNNING:
                check_policy_on_container(container.as_dict(),
                                          "container:stop")
                LOG.debug('Calling compute.container_stop with %s '
                          'before delete', container.uuid)
                compute_api.container_stop(context, container, 10)
        container.status = consts.DELETING
        if container.host:
            compute_api.container_delete(context, container, force)
        else:
            container.destroy(context)
        pecan.response.status = 204

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def rebuild(self, container_ident, **kwargs):
        """Rebuild container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:rebuild")
        utils.validate_container_state(container, 'rebuild')
        if kwargs.get('image'):
            container.image = kwargs.get('image')
        if kwargs.get('image_driver'):
            utils.validate_image_driver(kwargs.get('image_driver'))
            container.image_driver = kwargs.get('image_driver')
        LOG.debug('Calling compute.container_rebuild with %s',
                  container.uuid)
        run = True if container.status == consts.RUNNING else False
        context = pecan.request.context
        container.status = consts.REBUILDING
        container.save(context)
        compute_api = pecan.request.compute_api
        compute_api.container_rebuild(context, container, run)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def start(self, container_ident, **kwargs):
        """Start container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:start")
        utils.validate_container_state(container, 'start')
        LOG.debug('Calling compute.container_start with %s',
                  container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_start(context, container)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_stop)
    def stop(self, container_ident, timeout=None, **kwargs):
        """Stop container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:stop")
        utils.validate_container_state(container, 'stop')
        LOG.debug('Calling compute.container_stop with %s',
                  container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_stop(context, container, timeout)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_reboot)
    def reboot(self, container_ident, timeout=None, **kwargs):
        """Reboot container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:reboot")
        utils.validate_container_state(container, 'reboot')
        LOG.debug('Calling compute.container_reboot with %s',
                  container.uuid)
        context = pecan.request.context
        container.status = consts.RESTARTING
        container.save(context)
        compute_api = pecan.request.compute_api
        compute_api.container_reboot(context, container, timeout)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def pause(self, container_ident, **kwargs):
        """Pause container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:pause")
        utils.validate_container_state(container, 'pause')
        LOG.debug('Calling compute.container_pause with %s',
                  container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_pause(context, container)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def unpause(self, container_ident, **kwargs):
        """Unpause container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:unpause")
        utils.validate_container_state(container, 'unpause')
        LOG.debug('Calling compute.container_unpause with %s',
                  container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_unpause(context, container)
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_logs)
    def logs(self, container_ident, stdout=True, stderr=True,
             timestamps=False, tail='all', since=None):
        """Get logs of the given container.

        :param container_ident: UUID or Name of a container.
        :param stdout: Get standard output if True.
        :param stderr: Get standard error if True.
        :param timestamps: Show timestamps.
        :param tail: Number of lines to show from the end of the logs.
                     (default: get all logs)
        :param since: Show logs since a given datetime or
                     integer epoch (in seconds).
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:logs")
        utils.validate_container_state(container, 'logs')
        try:
            stdout = strutils.bool_from_string(stdout, strict=True)
            stderr = strutils.bool_from_string(stderr, strict=True)
            timestamps = strutils.bool_from_string(timestamps, strict=True)
        except ValueError:
            bools = ', '.join(strutils.TRUE_STRINGS + strutils.FALSE_STRINGS)
            raise exception.InvalidValue(_('Valid stdout, stderr and '
                                           'timestamps values are: %s')
                                         % bools)
        LOG.debug('Calling compute.container_logs with %s', container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        return compute_api.container_logs(context, container, stdout, stderr,
                                          timestamps, tail, since)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request,
                                     schema.query_param_execute_command)
    def execute(self, container_ident, run=True, interactive=False, **kwargs):
        """Execute command in a running container.

        :param container_ident: UUID or Name of a container.
        :param run: If True, execute run.
        :param interactive: Keep STDIN open and allocate a
                            pseudo-TTY for interactive.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:execute")
        utils.validate_container_state(container, 'execute')
        try:
            run = strutils.bool_from_string(run, strict=True)
            interactive = strutils.bool_from_string(interactive, strict=True)
        except ValueError:
            bools = ', '.join(strutils.TRUE_STRINGS + strutils.FALSE_STRINGS)
            raise exception.InvalidValue(_('Valid run or interactive '
                                           'values are: %s') % bools)
        LOG.debug('Calling compute.container_exec with %(uuid)s command '
                  '%(command)s',
                  {'uuid': container.uuid, 'command': kwargs['command']})
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        return compute_api.container_exec(context, container,
                                          kwargs['command'],
                                          run, interactive)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request,
                                     schema.query_param_execute_resize)
    def execute_resize(self, container_ident, exec_id, **kwargs):
        """Resize the tty session used by the exec

        :param container_ident: UUID or Name of a container.
        :param exec_id: ID of a exec.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(),
                                  "container:execute_resize")
        utils.validate_container_state(container, 'execute_resize')
        LOG.debug('Calling tty resize used by exec %s', exec_id)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        return compute_api.container_exec_resize(
            context, container, exec_id, kwargs.get('h', None),
            kwargs.get('w', None))

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validated(schema.query_param_signal)
    def kill(self, container_ident, **kwargs):
        """Kill a running container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:kill")
        utils.validate_container_state(container, 'kill')
        LOG.debug('Calling compute.container_kill with %(uuid)s '
                  'signal %(signal)s',
                  {'uuid': container.uuid,
                   'signal': kwargs.get('signal')})
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_kill(context, container, kwargs.get('signal'))
        pecan.response.status = 202

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def attach(self, container_ident):
        """Attach to a running container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:attach")
        utils.validate_container_state(container, 'attach')
        LOG.debug('Checking the status for attach with %s', container.uuid)
        if container.interactive:
            context = pecan.request.context
            compute_api = pecan.request.compute_api
            url = compute_api.container_attach(context, container)
            return url
        msg = _("Container doesn't support to be attached, "
                "please check the interactive set properly")
        raise exception.NoInteractiveFlag(msg=msg)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_resize)
    def resize(self, container_ident, **kwargs):
        """Resize container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:resize")
        utils.validate_container_state(container, 'resize')
        LOG.debug('Calling tty resize with %s ', container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_resize(context, container, kwargs.get('h', None),
                                     kwargs.get('w', None))

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_top)
    def top(self, container_ident, ps_args=None):
        """Display the running processes inside the container.

        :param container_ident: UUID or Name of a container.
        :param ps_args: The args of the ps command.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:top")
        utils.validate_container_state(container, 'top')
        LOG.debug('Calling compute.container_top with %s', container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        return compute_api.container_top(context, container, ps_args)

    @base.Controller.api_version("1.1", "1.24")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_archive(self, container_ident, **kwargs):
        """Retrieve a file/folder from a container

        Retrieve a file or folder from a container in the
        form of a tar archive.
        :param container_ident: UUID or Name of a container.
        """
        kwargs['encode_data'] = False
        return self._get_archive(container_ident, **kwargs)

    @base.Controller.api_version("1.25")  # noqa
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def get_archive(self, container_ident, **kwargs):
        """Retrieve a file/folder from a container

        Retrieve a file or folder from a container in the
        form of a tar archive.
        :param container_ident: UUID or Name of a container.
        """
        kwargs['encode_data'] = True
        return self._get_archive(container_ident, **kwargs)

    def _get_archive(self, container_ident, **kwargs):
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:get_archive")
        utils.validate_container_state(container, 'get_archive')
        LOG.debug('Calling compute.container_get_archive with %(uuid)s '
                  'path %(path)s',
                  {'uuid': container.uuid, 'path': kwargs['path']})
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        data, stat = compute_api.container_get_archive(
            context, container, kwargs['path'], kwargs['encode_data'])
        return {"data": data, "stat": stat}

    @base.Controller.api_version("1.1", "1.24")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def put_archive(self, container_ident, **kwargs):
        """Insert a file/folder to container.

        Insert a file or folder to an existing container using
        a tar archive as source.
        :param container_ident: UUID or Name of a container.
        """
        kwargs['decode_data'] = False
        self._put_archive(container_ident, **kwargs)

    @base.Controller.api_version("1.25")  # noqa
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def put_archive(self, container_ident, **kwargs):
        """Insert a file/folder to container.

        Insert a file or folder to an existing container using
        a tar archive as source.
        :param container_ident: UUID or Name of a container.
        """
        kwargs['decode_data'] = True
        self._put_archive(container_ident, **kwargs)

    def _put_archive(self, container_ident, **kwargs):
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:put_archive")
        utils.validate_container_state(container, 'put_archive')
        LOG.debug('Calling compute.container_put_archive with %(uuid)s '
                  'path %(path)s',
                  {'uuid': container.uuid, 'path': kwargs['path']})
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        compute_api.container_put_archive(
            context, container, kwargs['path'], kwargs['data'],
            kwargs['decode_data'])

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def stats(self, container_ident):
        """Display stats snapshot of the container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:stats")
        utils.validate_container_state(container, 'stats')
        LOG.debug('Calling compute.container_stats with %s', container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        return compute_api.container_stats(context, container)

    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.query_param_commit)
    def commit(self, container_ident, **kwargs):
        """Create a new image from a container's changes.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(), "container:commit")
        utils.validate_container_state(container, 'commit')
        LOG.debug('Calling compute.container_commit %s ', container.uuid)
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        pecan.response.status = 202
        return compute_api.container_commit(context, container,
                                            kwargs.get('repository', None),
                                            kwargs.get('tag', None))

    @base.Controller.api_version("1.6")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.network_detach)
    def network_detach(self, container_ident, **kwargs):
        """Detach a network from the container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(),
                                  "container:network_detach")
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        neutron_api = neutron.NeutronAPI(context)
        if kwargs.get('port'):
            port = neutron_api.get_neutron_port(kwargs['port'])
            net_id = port['network_id']
            if (net_id not in container.addresses or
                    port['id'] not in [a['port']
                                       for a in container.addresses[net_id]]):
                raise exception.Invalid(_(
                    "Port '%(port)s' is not attached to container "
                    "'%(container)s'.") %
                    {"port": kwargs.get('port'),
                     "container": container_ident})
        else:
            network = neutron_api.get_neutron_network(kwargs.get('network'))
            net_id = network['id']
            if net_id not in container.addresses:
                raise exception.Invalid(_(
                    "Network '%(network)s' is not attached to container "
                    "'%(container)s'.") %
                    {"network": kwargs.get('network'),
                     "container": container_ident})
        compute_api.network_detach(context, container, net_id)
        pecan.response.status = 202

    @base.Controller.api_version("1.8")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    @validation.validate_query_param(pecan.request, schema.network_attach)
    @validation.validated(schema.network_attach)
    def network_attach(self, container_ident, **kwargs):
        """Attach a network to the container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        check_policy_on_container(container.as_dict(),
                                  "container:network_attach")
        context = pecan.request.context
        compute_api = pecan.request.compute_api
        requested_networks = utils.build_requested_networks(context, [kwargs])
        if requested_networks[0]['network'] in container.addresses:
            if kwargs.get('port'):
                raise exception.Invalid(_(
                    "Cannot attach port '%(port)s' to container "
                    "'%(container)s' because another port on the "
                    "same network is already attached to this container.") %
                    {"port": kwargs.get('port'),
                     "container": container_ident})
            else:
                raise exception.Invalid(_(
                    "Network '%(network)s' is already connected to "
                    "container '%(container)s'.") %
                    {"network": kwargs.get('network'),
                     "container": container_ident})
        compute_api.network_attach(context, container, requested_networks[0])

    @base.Controller.api_version("1.13", "1.17")
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def network_list(self, container_ident):
        """Retrieve a list of networks of the container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        container_networks = self._get_container_networks_legacy(container)
        return {'networks': container_networks}

    def _get_container_networks_legacy(self, container):
        container_networks = []
        for net_id, net_infos in container.addresses.items():
            for net_info in net_infos:
                container_networks.append({
                    'net_id': net_id,
                    'subnet_id': net_info.get("subnet_id"),
                    'port_id': net_info.get("port"),
                    'version': net_info.get("version"),
                    'ip_address': net_info.get("addr")
                })
        return container_networks

    @base.Controller.api_version("1.18")  # noqa
    @pecan.expose('json')
    @exception.wrap_pecan_controller_exception
    def network_list(self, container_ident):
        """Retrieve a list of networks of the container.

        :param container_ident: UUID or Name of a container.
        """
        container = utils.get_container(container_ident)
        container_networks = self._get_container_networks(container)
        return {'networks': container_networks}

    def _get_container_networks(self, container):
        container_networks = []
        for net_id, net_infos in container.addresses.items():
            addresses = {}
            for net_info in net_infos:
                port_id = net_info["port"]
                addresses.setdefault(port_id, [])
                addresses[port_id].append({
                    'subnet_id': net_info.get("subnet_id"),
                    'version': net_info.get("version"),
                    'ip_address': net_info.get("addr")
                })
            for port_id, fixed_ips in addresses.items():
                container_networks.append({
                    'net_id': net_id,
                    'port_id': port_id,
                    'fixed_ips': fixed_ips,
                })
        return container_networks



#    Copyright 2016 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from zun.common import consts
from zun.common import exception
from zun.compute import api
from zun.compute import container_actions
import zun.conf
from zun import objects
from zun.tests import base
from zun.tests.unit.db import utils


CONF = zun.conf.CONF


class TestAPI(base.TestCase):

    def setUp(self):
        super(TestAPI, self).setUp()
        p = mock.patch('zun.scheduler.client.query.SchedulerClient')
        p.start()
        self.addCleanup(p.stop)

        self.compute_api = api.API(self.context)
        self.container = objects.Container(
            self.context, **utils.get_test_container())
        self.network = objects.Network(
            self.context, **utils.get_test_network())

    @mock.patch('zun.compute.api.API._record_action_start')
    @mock.patch('zun.compute.rpcapi.API.container_create')
    @mock.patch('zun.compute.rpcapi.API.image_search')
    @mock.patch('zun.compute.api.API._schedule_container')
    def test_container_create(self, mock_schedule_container,
                              mock_image_search,
                              mock_container_create,
                              mock_record_action_start):
        CONF.set_override('enable_image_validation', True, group="api")
        container = self.container
        container.status = consts.CREATING
        image_meta = mock.MagicMock()
        image_meta.id = '1234'
        mock_schedule_container.return_value = {'host': u'Centos',
                                                'nodename': None,
                                                'limits': {}}
        mock_image_search.return_value = [image_meta]
        self.compute_api.container_create(self.context, container,
                                          None, None, None, False)
        self.assertTrue(mock_schedule_container.called)
        self.assertTrue(mock_image_search.called)
        self.assertTrue(mock_container_create.called)

    @mock.patch('zun.compute.api.API._record_action_start')
    @mock.patch('zun.compute.rpcapi.API.container_create')
    @mock.patch('zun.compute.rpcapi.API.image_search')
    @mock.patch('zun.compute.api.API._schedule_container')
    def test_container_create_with_private_registry_image(
            self, mock_schedule_container, mock_image_search,
            mock_container_create, mock_record_action_start):
        CONF.set_override('enable_image_validation', True, group="api")
        container = self.container
        container.image = 'myregistry.io/test-image'
        container.image_driver = 'docker'
        mock_schedule_container.return_value = {'host': u'Centos',
                                                'nodename': None,
                                                'limits': {}}
        mock_image_search.side_effect = exception.OperationNotSupported

        self.compute_api.container_create(self.context, container,
                                          None, None, None, False)
        self.assertTrue(mock_schedule_container.called)
        self.assertTrue(mock_image_search.called)
        self.assertTrue(mock_container_create.called)

    @mock.patch('zun.compute.api.API._schedule_container')
    @mock.patch.object(objects.Container, 'save')
    def test_schedule_container_exception(self, mock_save,
                                          mock_schedule_container):
        container = self.container
        container.status = consts.CREATING
        mock_schedule_container.side_effect = exception.NoValidHost(
            reason='not enough host')
        self.compute_api.container_create(self.context, container,
                                          None, None, None, False)
        self.assertTrue(mock_schedule_container.called)
        self.assertTrue(mock_save.called)
        self.assertEqual(consts.ERROR, container.status)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    @mock.patch('zun.compute.rpcapi.API.image_search')
    @mock.patch('zun.compute.api.API._schedule_container')
    @mock.patch.object(objects.Container, 'save')
    def test_searching_image_exception(self, mock_save,
                                       mock_schedule_container,
                                       mock_image_search,
                                       mock_start, mock_cast):
        CONF.set_override('enable_image_validation', True, group="api")
        container = self.container
        container.status = consts.CREATING
        mock_schedule_container.return_value = {'host': u'Centos',
                                                'nodename': None,
                                                'limits': {}}
        mock_image_search.side_effect = exception.ZunException

        self.compute_api.container_create(
            self.context, container, None, None, None, False)
        self.assertTrue(mock_schedule_container.called)
        self.assertTrue(mock_image_search.called)
        self.assertFalse(mock_save.called)
        self.assertEqual(consts.CREATING, container.status)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_delete(self, mock_start, mock_srv_list,
                              mock_srv_up, mock_cast):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_delete(
            self.context, container, False)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.DELETE, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_delete",
            container=container, force=False)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_show(self, mock_srv_list,
                            mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_show(
            self.context, container)
        mock_call.assert_called_once_with(
            container.host, "container_show",
            container=container)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_reboot(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_reboot(self.context, container, 10)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.REBOOT, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_reboot",
            container=container, timeout=10)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_stop(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_stop(
            self.context, container, 10)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.STOP, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_stop", container=container, timeout=10)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_start(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_start(
            self.context, container)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.START, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_start", container=container)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_pause(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_pause(
            self.context, container)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.PAUSE, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_pause", container=container)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_unpause(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_unpause(
            self.context, container)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.UNPAUSE, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_unpause", container=container)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_logs(self, mock_srv_list,
                            mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_logs(
            self.context, container, 1, 1, 1, 1, 1)
        mock_call.assert_called_once_with(
            container.host, "container_logs",
            container=container, stdout=1, stderr=1,
            timestamps=1, tail=1, since=1)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_exec(self, mock_srv_list,
                            mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_exec(
            self.context, container, "/bin/bash", True, True)
        mock_call.assert_called_once_with(
            container.host, "container_exec",
            container=container, command="/bin/bash",
            run=True, interactive=True)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_exec_interactive(
            self, mock_srv_list, mock_srv_up, mock_call):
        mock_call.return_value = {'token': 'fake-token',
                                  'exec_id': 'fake-exec-id'}
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        result = self.compute_api.container_exec(
            self.context, container, "/bin/bash", True, True)
        self.assertIn('fake-token', result['proxy_url'])
        self.assertIn('fake-exec-id', result['proxy_url'])
        mock_call.assert_called_once_with(
            container.host, "container_exec",
            container=container, command="/bin/bash",
            run=True, interactive=True)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_exec_resize(self, mock_srv_list,
                                   mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_exec_resize(
            self.context, container, '123', 10, 5)
        mock_call.assert_called_once_with(
            container.host, "container_exec_resize",
            exec_id='123', height=10, width=5)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_kill(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.container_kill(
            self.context, container, 9)
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.KILL, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "container_kill", container=container, signal=9)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_update(self, mock_srv_list,
                              mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_update(
            self.context, container, {})
        mock_call.assert_called_once_with(
            container.host, "container_update",
            container=container, patch={})

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_attach(self, mock_srv_list,
                              mock_srv_up, mock_call):
        mock_call.return_value = 'fake-token'
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        url = self.compute_api.container_attach(self.context, container)
        mock_call.assert_called_once_with(
            container.host, "container_attach",
            container=container)
        expected_url = '%s?token=%s&uuid=%s' % (
            CONF.websocket_proxy.base_url, 'fake-token', container.uuid)
        self.assertEqual(expected_url, url)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_resize(self, mock_srv_list,
                              mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_resize(self.context, container, 10, 5)
        mock_call.assert_called_once_with(
            container.host, "container_resize",
            container=container, height=10, width=5)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_top(self, mock_srv_list,
                           mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_top(self.context, container, "")
        mock_call.assert_called_once_with(
            container.host, "container_top",
            container=container, ps_args="")

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_get_archive(self, mock_srv_list,
                                   mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_get_archive(
            self.context, container, "/root", True)
        mock_call.assert_called_once_with(
            container.host, "container_get_archive",
            container=container, path="/root", encode_data=True)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_add_security_group(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.add_security_group(
            self.context, container, {})
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.ADD_SECURITY_GROUP, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "add_security_group",
            container=container, security_group={})

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_remove_security_group(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.remove_security_group(
            self.context, container, {})
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.REMOVE_SECURITY_GROUP, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "remove_security_group",
            container=container, security_group={})

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_put_archive(self, mock_srv_list,
                                   mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_put_archive(
            self.context, container, "/root", {}, True)
        mock_call.assert_called_once_with(
            container.host, "container_put_archive",
            container=container, path="/root", data={}, decode_data=True)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    def test_container_stats(self, mock_srv_list,
                             mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_stats(self.context, container)
        mock_call.assert_called_once_with(
            container.host, "container_stats",
            container=container)

    @mock.patch('zun.compute.rpcapi.API._call')
    @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
    @mock.patch('zun.objects.ZunService.list_by_binary')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_container_commit(self, mock_start, mock_srv_list,
                              mock_srv_up, mock_call):
        container = self.container
        srv = objects.ZunService(
            self.context,
            **utils.get_test_zun_service(host=container.host))
        mock_srv_list.return_value = [srv]
        mock_srv_up.return_value = True
        self.compute_api.container_commit(
            self.context, container, "ubuntu", "latest")
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.COMMIT, want_result=False)
        mock_call.assert_called_once_with(
            container.host, "container_commit",
            container=container, repository="ubuntu", tag="latest")

    @mock.patch('zun.compute.rpcapi.API._call')
    def test_image_search(self, mock_call):
        self.compute_api.image_search(
            self.context, "ubuntu", "glance", True, None)
        mock_call.assert_called_once_with(
            None, "image_search", image="ubuntu", registry=None,
            image_driver_name="glance", exact_match=True)

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_network_attach(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.network_attach(
            self.context, container, {})
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.NETWORK_ATTACH, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "network_attach", container=container,
            requested_network={})

    @mock.patch('zun.compute.rpcapi.API._cast')
    @mock.patch.object(objects.ContainerAction, 'action_start')
    def test_network_detach(self, mock_start, mock_cast):
        container = self.container
        self.compute_api.network_detach(
            self.context, container, {})
        mock_start.assert_called_once_with(
            self.context, container.uuid,
            container_actions.NETWORK_DETACH, want_result=False)
        mock_cast.assert_called_once_with(
            container.host, "network_detach", container=container, network={})

    @mock.patch('zun.compute.rpcapi.API.network_create')
    def test_network_create(self, mock_network_create):
        network = self.network
        self.compute_api.network_create(self.context, network)
        self.assertTrue(mock_network_create.called)



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from zun.api import hooks

# Pecan Application Configurations
app = {
    'root': 'zun.api.controllers.root.RootController',
    'modules': ['zun'],
    'hooks': [
        hooks.ContextHook(),
        hooks.NoExceptionTracebackHook(),
        hooks.RPCHook(),
    ],
    'debug': False,
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Zun test utilities."""

from oslo_config import cfg

from zun.common import consts
from zun.common import name_generator
from zun.db import api as db_api
from zun.objects.container import Cpuset

CONF = cfg.CONF


def get_cpuset_obj():
    return Cpuset._from_dict({'cpuset_cpus': set([0, 1]),
                              'cpuset_mems': set([0])})


def get_cpuset_dict():
    return {'cpuset_cpus': set([0, 1]), 'cpuset_mems': set([0])}


def get_test_container(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'uuid': kwargs.get('uuid', 'ea8e2a25-2901-438d-8157-de7ffd68d051'),
        'container_id': kwargs.get('container_id', 'ddcb39a3fcec'),
        'name': kwargs.get('name', 'container1'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'image': kwargs.get('image', 'ubuntu'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'command': kwargs.get('command', ['fake_command']),
        'status': kwargs.get('status', 'Running'),
        'status_reason': kwargs.get('status_reason', 'Created Successfully'),
        'task_state': kwargs.get('task_state', None),
        'environment': kwargs.get('environment', {'key1': 'val1',
                                                  'key2': 'val2'}),
        'cpu': kwargs.get('cpu', 1.0),
        'memory': kwargs.get('memory', '512'),
        'workdir': kwargs.get('workdir', '/home/ubuntu'),
        'ports': kwargs.get('ports', [80, 443]),
        'hostname': kwargs.get('hostname', 'testhost'),
        'labels': kwargs.get('labels', {'key1': 'val1', 'key2': 'val2'}),
        'meta': kwargs.get('meta', {'key1': 'val1', 'key2': 'val2'}),
        'addresses': kwargs.get('addresses', {
            kwargs.get('network', 'private'): [
                {
                    'subnet_id': 'f89ae741-999e-4873-b38c-779e3deb8458',
                    'version': 4,
                    'preserve_on_delete': False,
                    'addr': '172.24.4.4',
                    'port': kwargs.get('port',
                                       '22626847-f511-42ee-ab06-8a9764ad2680')
                },
                {
                    'subnet_id': '3e4e9708-d83b-46fb-8591-8143bd66206e',
                    'version': 6,
                    'preserve_on_delete': False,
                    'addr': '2001:db8::5',
                    'port': kwargs.get('port',
                                       '22626847-f511-42ee-ab06-8a9764ad2680')
                }
            ],
        }),
        'image_pull_policy': kwargs.get('image_pull_policy', 'always'),
        'host': kwargs.get('host', 'localhost'),
        'restart_policy': kwargs.get('restart_policy',
                                     {'Name': 'no', 'MaximumRetryCount': '0'}),
        'status_detail': kwargs.get('status_detail', 'up from 5 hours'),
        'interactive': kwargs.get('interactive', True),
        'tty': kwargs.get('tty', True),
        'image_driver': kwargs.get('image_driver', 'glance'),
        'websocket_url': 'ws://127.0.0.1:6784/4c03164962fa/attach/'
                         'ws?logs=0&stream=1&stdin=1&stdout=1&stderr=1',
        'websocket_token': '7878038e-957c-4d52-ae19-1e9561784e7b',
        'security_groups': kwargs.get('security_groups', ['default']),
        'auto_remove': kwargs.get('auto_remove', False),
        'runtime': kwargs.get('runtime', 'runc'),
        'disk': kwargs.get('disk', 20),
        'auto_heal': kwargs.get('auto_heal', False),
        'started_at': kwargs.get('started_at'),
        'privileged': kwargs.get('privileged', False),
        'healthcheck': kwargs.get('healthcheck',
                                  {"retries": "2", "timeout": 3,
                                   "test": "stat /etc/passwd || exit 1",
                                   "interval": 3}),
        'exposed_ports': kwargs.get('exposed_ports', {"80/tcp": {}}),
        'cpu_policy': kwargs.get('cpu_policy', None),
        'cpuset': kwargs.get('cpuset', None),
        'registry_id': kwargs.get('registry_id', None),
        'container_type': kwargs.get('container_type', consts.TYPE_CONTAINER),
        'capsule_id': kwargs.get('capsule_id', 33),
    }


def _get_dbapi():
    dbapi = db_api._get_dbdriver_instance()
    return dbapi


def create_test_container(**kwargs):
    """Create test container entry in DB and return Container DB object.

    Function to be used to create test Container objects in the database.
    :param kwargs: kwargs with overriding values for container's attributes.
    :returns: Test Container DB object.
    """
    container = get_test_container(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del container['id']
    dbapi = _get_dbapi()
    return dbapi.create_container(kwargs['context'], container)


def get_test_volume_mapping(**kwargs):
    return {
        'id': kwargs.get('id', 38),
        'uuid': kwargs.get('uuid', 'bea6293e-7714-4cd2-abc5-d479a498cee3'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'container_path': kwargs.get('container_path', 'fake_path'),
        'container_uuid': kwargs.get('container_uuid',
                                     '1aca1705-20f3-4506-8bc3-59685d86a357'),
        'volume_id': kwargs.get('volume_id', 39),
    }


def create_test_volume_mapping(**kwargs):
    volume_mapping = get_test_volume_mapping(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del volume_mapping['id']
    dbapi = _get_dbapi()
    return dbapi.create_volume_mapping(kwargs['context'], volume_mapping)


def get_test_volume(**kwargs):
    return {
        'id': kwargs.get('id', 39),
        'uuid': kwargs.get('uuid', 'c0aae414-4462-45ae-9848-1312983d1f7a'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'cinder_volume_id': kwargs.get('cinder_volume_id',
                                       '342a140e-efca-4140-9d2a-64221f688fa2'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'volume_provider': kwargs.get('volume_provider', 'fake_provider'),
        'connection_info': kwargs.get('connection_info', 'fake_info'),
        'auto_remove': kwargs.get('auto_remove', False),
        'host': kwargs.get('host', 'fake_host'),
        'contents': kwargs.get('contents', 'fake-contents'),
    }


def create_test_volume(**kwargs):
    volume = get_test_volume(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del volume['id']
    dbapi = _get_dbapi()
    return dbapi.create_volume(kwargs['context'], volume)


def get_test_image(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'uuid': kwargs.get('uuid', 'ea8e2a25-2901-438d-8157-de7ffd68d051'),
        'repo': kwargs.get('repo', 'image1'),
        'tag': kwargs.get('tag', 'latest'),
        'image_id': kwargs.get('image_id', 'sha256:c54a2cc56cbb2f0400'),
        'size': kwargs.get('size', '1848'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'host': kwargs.get('host', 'host1'),
    }


def create_test_image(**kwargs):
    """Create test image entry in DB and return Image DB object.

    Function to be used to create test Image objects in the database.
    :param kwargs: kwargs with overriding values for image's attributes.
    :returns: Test Image DB object.
    """
    image = get_test_image(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del image['id']
    if 'repo' not in kwargs:
        image['repo'] = _generate_repo_for_image()
    dbapi = _get_dbapi()
    return dbapi.pull_image(kwargs['context'], image)


def _generate_repo_for_image():
    """Generate a random name like: zeta-22-image."""
    name_gen = name_generator.NameGenerator()
    name = name_gen.generate()
    return name + '-image'


def get_test_zun_service(**kwargs):
    return {
        'id': kwargs.get('id', 23),
        'uuid': kwargs.get('uuid', '2e8e2a25-2901-438d-8157-de7ffd68d066'),
        'host': kwargs.get('host', 'fakehost'),
        'binary': kwargs.get('binary', 'fake-bin'),
        'disabled': kwargs.get('disabled', False),
        'disabled_reason': kwargs.get('disabled_reason', 'fake-reason'),
        'last_seen_up': kwargs.get('last_seen_up'),
        'forced_down': kwargs.get('forced_down', False),
        'report_count': kwargs.get('report_count', 13),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'availability_zone': kwargs.get('availability_zone', 'fake-zone'),
    }


def create_test_zun_service(**kwargs):
    zun_service = get_test_zun_service(**kwargs)
    # Let DB generate ID if it isn't specifiled explicitly
    if 'id' not in kwargs:
        del zun_service['id']
    dbapi = _get_dbapi()
    return dbapi.create_zun_service(zun_service)


def get_test_resource_provider(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'uuid': kwargs.get('uuid', 'e166bf0e-66db-409d-aa4d-3af94efd8bcf'),
        'name': kwargs.get('name', 'provider1'),
        'root_provider': kwargs.get('root_provider',
                                    'd3d4c98a-8c95-4d3c-8605-ea38ea036556'),
        'parent_provider': kwargs.get('parent_provider',
                                      '2c4de408-6c4f-4257-8274-f2d2192fe871'),
        'can_host': kwargs.get('can_host', 0),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
    }


def create_test_resource_provider(**kwargs):
    provider = get_test_resource_provider(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del provider['id']
    dbapi = _get_dbapi()
    return dbapi.create_resource_provider(kwargs['context'], provider)


def get_test_resource_class(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'uuid': kwargs.get('uuid', '1136bf0e-66db-409d-aa4d-3af94eed8bcc'),
        'name': kwargs.get('name', 'VCPU'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
    }


def create_test_resource_class(**kwargs):
    resource = get_test_resource_class(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del resource['id']
    dbapi = _get_dbapi()
    return dbapi.create_resource_class(kwargs['context'], resource)


def get_test_inventory(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'resource_provider_id': kwargs.get('resource_provider_id', 1),
        'resource_class_id': kwargs.get('resource_class_id', 2),
        'total': kwargs.get('total', 4),
        'reserved': kwargs.get('reserved', 1),
        'min_unit': kwargs.get('min_unit', 0),
        'max_unit': kwargs.get('max_unit', 4),
        'step_size': kwargs.get('step_size', 1),
        'allocation_ratio': kwargs.get('allocation_ratio', 1.0),
        'is_nested': kwargs.get('is_nested', True),
        'blob': kwargs.get('blob', [1, 2, 5]),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
    }


def create_test_inventory(**kwargs):
    inventory = get_test_inventory(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del inventory['id']
    provider_id = inventory.pop('resource_provider_id')
    dbapi = _get_dbapi()
    return dbapi.create_inventory(kwargs['context'], provider_id, inventory)


def get_test_allocation(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'resource_provider_id': kwargs.get('resource_provider_id', 1),
        'resource_class_id': kwargs.get('resource_class_id', 2),
        'consumer_id': kwargs.get('consumer_id',
                                  'cb775e13-72e2-4bfa-9646-020725e1325e'),
        'used': kwargs.get('used', 1),
        'is_nested': kwargs.get('is_nested', 0),
        'blob': kwargs.get('blob', [1, 2, 5]),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
    }


def create_test_allocation(**kwargs):
    allocation = get_test_allocation(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del allocation['id']
    dbapi = _get_dbapi()
    return dbapi.create_allocation(kwargs['context'], allocation)


def get_test_numa_topology(**kwargs):
    return {
        "nodes": [
            {
                "id": 0,
                "cpuset": [1, 2],
                "pinned_cpus": [],
                "mem_total": 65536,
                "mem_available": 32768
            },
            {
                "id": 1,
                "cpuset": [3, 4],
                "pinned_cpus": [3, 4],
                "mem_total": 65536,
                "mem_available": 32768
            }
        ]
    }


def get_test_compute_node(**kwargs):
    return {
        'uuid': kwargs.get('uuid', '24a5b17a-f2eb-4556-89db-5f4169d13982'),
        'rp_uuid': kwargs.get('rp_uuid',
                              '24a5b17a-f2eb-4556-89db-5f4169d13982'),
        'hostname': kwargs.get('hostname', 'localhost'),
        'numa_topology': kwargs.get('numa_topology', get_test_numa_topology()),
        'mem_total': kwargs.get('mem_total', 1024),
        'mem_free': kwargs.get('mem_free', 512),
        'mem_available': kwargs.get('mem_available', 512),
        'mem_used': kwargs.get('mem_used', 512),
        'total_containers': kwargs.get('total_containers', 10),
        'running_containers': kwargs.get('running_containers', 8),
        'paused_containers': kwargs.get('paused_containers', 0),
        'stopped_containers': kwargs.get('stopped_containers', 2),
        'cpus': kwargs.get('cpus', 48),
        'cpu_used': kwargs.get('cpu_used', 6.5),
        'architecture': kwargs.get('architecture', 'x86_64'),
        'os_type': kwargs.get('os_type', 'linux'),
        'os': kwargs.get('os', 'Centos'),
        'kernel_version': kwargs.get('kernel_version',
                                     '3.10.0-123.el7.x86_64'),
        'labels': kwargs.get('labels', {"dev.type": "product"}),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'disk_total': kwargs.get('disk_total', 80),
        'disk_used': kwargs.get('disk_used', 20),
        'disk_quota_supported': kwargs.get('disk_quota_supported', False),
        'runtimes': kwargs.get('runtimes', ['runc']),
        'enable_cpu_pinning': kwargs.get('enable_cpu_pinning', False),
    }


def create_test_compute_node(**kwargs):
    compute_host = get_test_compute_node(**kwargs)
    dbapi = _get_dbapi()
    return dbapi.create_compute_node(kwargs['context'], compute_host)


class FakeObject(object):
    def __getitem__(self, key):
        return getattr(self, key)


def get_test_action_value(**kwargs):
    action_values = {
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'id': kwargs.get('id', 123),
        'action': kwargs.get('action', 'fake-action'),
        'container_uuid': kwargs.get('container_uuid',
                                     'ea8e2a25-2901-438d-8157-de7ffd68d051'),
        'request_id': kwargs.get('request_id', 'fake-request'),
        'user_id': kwargs.get('user_id', 'fake-user'),
        'project_id': kwargs.get('project_id', 'fake-project'),
        'start_time': kwargs.get('start_time'),
        'finish_time': kwargs.get('finish_time'),
        'message': kwargs.get('message', 'fake-message'),
    }

    return action_values


def get_test_action(**kwargs):

    action_values = get_test_action_value(**kwargs)
    fake_action = FakeObject()
    for k, v in action_values.items():
        setattr(fake_action, k, v)
    return fake_action


def get_test_action_event_value(**kwargs):

    event_values = {
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'id': kwargs.get('id', 123),
        'event': kwargs.get('event', 'fake-event'),
        'action_id': kwargs.get('action_id', 123),
        'start_time': kwargs.get('start_time'),
        'finish_time': kwargs.get('finish_time'),
        'result': kwargs.get('result', 'Error'),
        'traceback': kwargs.get('traceback', 'fake-tb'),
    }

    return event_values


def get_test_action_event(**kwargs):

    event_values = get_test_action_event_value(**kwargs)
    fake_event = FakeObject()
    for k, v in event_values.items():
        setattr(fake_event, k, v)
    return fake_event


def create_test_quota(**kwargs):
    context = kwargs.get('context')
    project_id = kwargs.get('project_id', 'fake_project_id')
    resource = kwargs.get('resource', 'containers')
    limit = kwargs.get('limit', 100)
    dbapi = _get_dbapi()
    return dbapi.quota_create(context, project_id, resource, limit)


def create_test_quota_class(**kwargs):
    context = kwargs.get('context')
    class_name = kwargs.get('class_name', 'default')
    resource = kwargs.get('resource', 'containers')
    limit = kwargs.get('limit', 100)
    dbapi = _get_dbapi()
    return dbapi.quota_class_create(context, class_name, resource, limit)


def get_test_quota_value(**kwargs):
    quota_values = {
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'id': kwargs.get('id', 123),
        'project_id': kwargs.get('project_id', 'fake_project_id'),
        'resource': kwargs.get('resource', 'container'),
        'hard_limit': kwargs.get('hard_limit', 20),
        'uuid': kwargs.get('uuid', 'z2b96c5f-242a-41a0-a736-b6e1fada071b'),
    }

    return quota_values


def get_test_quota(**kwargs):
    quota_values = get_test_quota_value(**kwargs)
    fake_quota = FakeObject()
    for k, v in quota_values.items():
        setattr(fake_quota, k, v)

    return fake_quota


def get_test_quota_class_value(**kwargs):
    quota_values = {
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'id': kwargs.get('id', 123),
        'class_name': kwargs.get('class_name', 'fake_class_name'),
        'resource': kwargs.get('resource', 'container'),
        'hard_limit': kwargs.get('hard_limit', 20),
        'uuid': kwargs.get('uuid', 'z2b96c5b-242a-41a0-a736-b6e1fada071b'),
    }

    return quota_values


def get_test_quota_class(**kwargs):
    quota_class_values = get_test_quota_class_value(**kwargs)
    fake_quota_class = FakeObject()
    for k, v in quota_class_values.items():
        setattr(fake_quota_class, k, v)

    return fake_quota_class


def get_test_network(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'name': kwargs.get('name', 'fake_name'),
        'uuid': kwargs.get('uuid', 'z2b96c5b-242a-41a0-a736-b6e1fada071b'),
        'network_id': kwargs.get('network_id', '0eeftestnetwork'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'neutron_net_id': kwargs.get('neutron_net_id', 'bar'),
    }


def get_test_exec_instance(**kwargs):
    return {
        'id': kwargs.get('id', 43),
        'container_id': kwargs.get('container_id', 42),
        'exec_id': kwargs.get('exec_id', 'fake-exec-id'),
        'token': kwargs.get('token', 'fake-exec-token'),
        'url': kwargs.get('url', 'fake-url'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
    }


def create_test_exec_instance(**kwargs):
    """Create test exec instance entry in DB and return ExecInstance DB object.

    Function to be used to create test ExecInstance objects in the database.
    :param kwargs: kwargs with overriding values for default attributes.
    :returns: Test ExecInstance DB object.
    """
    exec_inst = get_test_exec_instance(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del exec_inst['id']
    dbapi = _get_dbapi()
    return dbapi.create_exec_instance(kwargs['context'], exec_inst)


def create_test_network(**kwargs):
    network = get_test_network(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del network['id']
    dbapi = _get_dbapi()
    return dbapi.create_network(kwargs['context'], network)


def get_test_registry(**kwargs):
    return {
        'id': kwargs.get('id', 42),
        'name': kwargs.get('name', 'fake_name'),
        'uuid': kwargs.get('uuid', '0b5cdde8-237a-4917-9556-003e5c588c4f'),
        'project_id': kwargs.get('project_id', 'fake_project'),
        'user_id': kwargs.get('user_id', 'fake_user'),
        'created_at': kwargs.get('created_at'),
        'updated_at': kwargs.get('updated_at'),
        'domain': kwargs.get('domain', 'test.io'),
        'username': kwargs.get('username', 'fake_username'),
        'password': kwargs.get('password', 'fake_password'),
    }


def create_test_registry(**kwargs):
    registry = get_test_registry(**kwargs)
    # Let DB generate ID if it isn't specified explicitly
    if 'id' not in kwargs:
        del registry['id']
    dbapi = _get_dbapi()
    return dbapi.create_registry(kwargs['context'], registry)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from oslo_utils import uuidutils
from testtools.matchers import HasLength

from zun import objects
from zun.tests.unit.db import base
from zun.tests.unit.db import utils


class TestResourceProviderObject(base.DbTestCase):

    def setUp(self):
        super(TestResourceProviderObject, self).setUp()
        self.fake_provider = utils.get_test_resource_provider()

    def test_get_by_uuid(self):
        uuid = self.fake_provider['uuid']
        with mock.patch.object(self.dbapi, 'get_resource_provider',
                               autospec=True) as mock_get_resource_provider:
            mock_get_resource_provider.return_value = self.fake_provider
            provider = objects.ResourceProvider.get_by_uuid(self.context, uuid)
            mock_get_resource_provider.assert_called_once_with(
                self.context, uuid)
            self.assertEqual(self.context, provider._context)

    def test_get_by_name(self):
        name = self.fake_provider['name']
        with mock.patch.object(self.dbapi, 'get_resource_provider',
                               autospec=True) as mock_get_resource_provider:
            mock_get_resource_provider.return_value = self.fake_provider
            provider = objects.ResourceProvider.get_by_name(self.context, name)
            mock_get_resource_provider.assert_called_once_with(
                self.context, name)
            self.assertEqual(self.context, provider._context)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'list_resource_providers',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_provider]
            providers = objects.ResourceProvider.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(providers, HasLength(1))
            self.assertIsInstance(providers[0], objects.ResourceProvider)
            self.assertEqual(self.context, providers[0]._context)

    def test_list_with_filters(self):
        with mock.patch.object(self.dbapi, 'list_resource_providers',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_provider]
            filt = {'name': 'testprovider'}
            providers = objects.ResourceProvider.list(
                self.context, filters=filt)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(providers, HasLength(1))
            self.assertIsInstance(providers[0], objects.ResourceProvider)
            self.assertEqual(self.context, providers[0]._context)
            mock_get_list.assert_called_once_with(
                self.context, filters=filt, limit=None, marker=None,
                sort_key=None, sort_dir=None)

    def test_create(self):
        with mock.patch.object(self.dbapi, 'create_resource_provider',
                               autospec=True) as mock_create:
            mock_create.return_value = self.fake_provider
            provider = objects.ResourceProvider(
                self.context, **self.fake_provider)
            provider.create(self.context)
            mock_create.assert_called_once_with(
                self.context, self.fake_provider)
            self.assertEqual(self.context, provider._context)

    def test_destroy(self):
        uuid = self.fake_provider['uuid']
        with mock.patch.object(self.dbapi, 'get_resource_provider',
                               autospec=True) as mock_get_resource_provider:
            mock_get_resource_provider.return_value = self.fake_provider
            with mock.patch.object(self.dbapi, 'destroy_resource_provider',
                                   autospec=True) as mock_destroy:
                provider = objects.ResourceProvider.get_by_uuid(
                    self.context, uuid)
                provider.destroy()
                mock_get_resource_provider.assert_called_once_with(
                    self.context, uuid)
                mock_destroy.assert_called_once_with(None, uuid)
                self.assertEqual(self.context, provider._context)

    def test_save(self):
        uuid = self.fake_provider['uuid']
        with mock.patch.object(self.dbapi, 'get_resource_provider',
                               autospec=True) as mock_get_resource_provider:
            mock_get_resource_provider.return_value = self.fake_provider
            with mock.patch.object(self.dbapi, 'update_resource_provider',
                                   autospec=True) as mock_update:
                provider = objects.ResourceProvider.get_by_uuid(
                    self.context, uuid)
                provider.name = 'provider2'
                provider.root_provider = '09d0fcb9-155e-434a-ad76-3620b6382a37'
                provider.save()

                mock_get_resource_provider.assert_called_once_with(
                    self.context, uuid)
                mock_update.assert_called_once_with(
                    None, uuid,
                    {'name': 'provider2',
                     'root_provider': '09d0fcb9-155e-434a-ad76-3620b6382a37'})
                self.assertEqual(self.context, provider._context)

    def test_refresh(self):
        uuid = self.fake_provider['uuid']
        new_uuid = uuidutils.generate_uuid()
        returns = [dict(self.fake_provider, uuid=uuid),
                   dict(self.fake_provider, uuid=new_uuid)]
        expected = [mock.call(self.context, uuid),
                    mock.call(self.context, uuid)]
        with mock.patch.object(self.dbapi, 'get_resource_provider',
                               side_effect=returns,
                               autospec=True) as mock_get_resource_provider:
            provider = objects.ResourceProvider.get_by_uuid(self.context, uuid)
            self.assertEqual(uuid, provider.uuid)
            provider.refresh()
            self.assertEqual(new_uuid, provider.uuid)
            self.assertEqual(
                expected, mock_get_resource_provider.call_args_list)
            self.assertEqual(self.context, provider._context)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating Containers via the DB API"""

from oslo_utils import uuidutils
import six

from zun.common import consts
from zun.common import exception
import zun.conf
from zun.db import api as dbapi
from zun.tests.unit.db import base
from zun.tests.unit.db import utils

CONF = zun.conf.CONF


class DbContainerTestCase(base.DbTestCase):

    def setUp(self):
        super(DbContainerTestCase, self).setUp()

    def test_create_container(self):
        utils.create_test_container(context=self.context)

    def test_create_container_already_exists(self):
        CONF.set_override("unique_container_name_scope", "",
                          group="compute")
        utils.create_test_container(context=self.context,
                                    uuid='123')
        with self.assertRaisesRegex(exception.ContainerAlreadyExists,
                                    'A container with UUID 123.*'):
            utils.create_test_container(context=self.context,
                                        uuid='123')

    def test_create_container_already_exists_in_project_name_space(self):
        CONF.set_override("unique_container_name_scope", "project",
                          group="compute")
        utils.create_test_container(context=self.context, name='cont1')
        with self.assertRaisesRegex(exception.ContainerAlreadyExists,
                                    'A container with name.*'):
            utils.create_test_container(uuid=uuidutils.generate_uuid(),
                                        context=self.context,
                                        name='cont1')
        utils.create_test_container(context=self.context,
                                    name='abc',
                                    uuid=uuidutils.generate_uuid())

    def test_create_container_already_exists_in_global_name_space(self):
        CONF.set_override("unique_container_name_scope", "global",
                          group="compute")
        utils.create_test_container(context=self.context, name='cont1')
        self.context.project_id = 'fake_project_1'
        self.context.user_id = 'fake_user_1'
        with self.assertRaisesRegex(exception.ContainerAlreadyExists,
                                    'A container with name.*'):
            utils.create_test_container(uuid=uuidutils.generate_uuid(),
                                        context=self.context,
                                        name='cont1')
        utils.create_test_container(context=self.context,
                                    name='abc',
                                    uuid=uuidutils.generate_uuid())

    def test_create_container_already_exists_in_default_name_space(self):
        CONF.set_override("unique_container_name_scope", "",
                          group="compute")
        utils.create_test_container(context=self.context,
                                    name='cont1',
                                    uuid=uuidutils.generate_uuid())
        self.context.project_id = 'fake_project_1'
        self.context.user_id = 'fake_user_1'
        utils.create_test_container(context=self.context,
                                    name='cont1',
                                    uuid=uuidutils.generate_uuid())
        utils.create_test_container(context=self.context,
                                    name='abc',
                                    uuid=uuidutils.generate_uuid())

    def test_get_container_by_uuid(self):
        container = utils.create_test_container(context=self.context)
        res = dbapi.get_container_by_uuid(self.context,
                                          container.container_type,
                                          container.uuid)
        self.assertEqual(container.id, res.id)
        self.assertEqual(container.uuid, res.uuid)

    def test_get_container_by_name(self):
        container = utils.create_test_container(context=self.context)
        res = dbapi.get_container_by_name(
            self.context, container.container_type, container.name)
        self.assertEqual(container.id, res.id)
        self.assertEqual(container.uuid, res.uuid)

    def test_get_container_that_does_not_exist(self):
        self.assertRaises(exception.ContainerNotFound,
                          dbapi.get_container_by_uuid,
                          self.context,
                          consts.TYPE_CONTAINER,
                          uuidutils.generate_uuid())

    def test_list_containers(self):
        uuids = []
        for i in range(1, 6):
            container = utils.create_test_container(
                uuid=uuidutils.generate_uuid(),
                context=self.context,
                name='container' + str(i))
            uuids.append(six.text_type(container['uuid']))
        res = dbapi.list_containers(self.context, consts.TYPE_CONTAINER)
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), sorted(res_uuids))

    def test_list_containers_sorted(self):
        uuids = []
        for i in range(5):
            container = utils.create_test_container(
                uuid=uuidutils.generate_uuid(),
                context=self.context,
                name='container' + str(i))
            uuids.append(six.text_type(container.uuid))
        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER, sort_key='uuid')
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), res_uuids)

        self.assertRaises(exception.InvalidParameterValue,
                          dbapi.list_containers,
                          self.context,
                          consts.TYPE_CONTAINER,
                          sort_key='foo')

    def test_list_containers_with_filters(self):
        container1 = utils.create_test_container(
            name='container-one',
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        container2 = utils.create_test_container(
            name='container-two',
            uuid=uuidutils.generate_uuid(),
            context=self.context)

        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER,
            filters={'name': 'container-one'})
        self.assertEqual([container1.id], [r.id for r in res])

        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER,
            filters={'name': 'container-two'})
        self.assertEqual([container2.id], [r.id for r in res])

        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER,
            filters={'name': 'bad-container'})
        self.assertEqual([], [r.id for r in res])

        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER,
            filters={'name': container1.name})
        self.assertEqual([container1.id], [r.id for r in res])

    def test_list_containers_with_list_filters(self):
        container1 = utils.create_test_container(
            name='container-one',
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        container2 = utils.create_test_container(
            name='container-two',
            uuid=uuidutils.generate_uuid(),
            context=self.context)

        res = dbapi.list_containers(
            self.context, consts.TYPE_CONTAINER,
            filters={'name': ['container-one', 'container-two']})
        uuids = sorted([container1.uuid, container2.uuid])
        self.assertEqual(uuids, sorted([r.uuid for r in res]))

    def test_destroy_container(self):
        container = utils.create_test_container(context=self.context)
        dbapi.destroy_container(self.context, container.container_type,
                                container.id)
        self.assertRaises(exception.ContainerNotFound,
                          dbapi.get_container_by_uuid,
                          self.context, container.container_type,
                          container.uuid)

    def test_destroy_container_by_uuid(self):
        container = utils.create_test_container(context=self.context)
        dbapi.destroy_container(self.context, container.container_type,
                                container.uuid)
        self.assertRaises(exception.ContainerNotFound,
                          dbapi.get_container_by_uuid,
                          self.context, container.container_type,
                          container.uuid)

    def test_destroy_container_that_does_not_exist(self):
        self.assertRaises(exception.ContainerNotFound,
                          dbapi.destroy_container, self.context,
                          consts.TYPE_CONTAINER,
                          uuidutils.generate_uuid())

    def test_update_container(self):
        container = utils.create_test_container(context=self.context)
        old_image = container.image
        new_image = 'new-image'
        self.assertNotEqual(old_image, new_image)

        res = dbapi.update_container(self.context, container.container_type,
                                     container.id,
                                     {'image': new_image})
        self.assertEqual(new_image, res.image)

    def test_update_container_with_the_same_name(self):
        CONF.set_override("unique_container_name_scope", "project",
                          group="compute")
        container1 = utils.create_test_container(
            name='container-one',
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        container2 = utils.create_test_container(
            name='container-two',
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        new_name = 'new_name'
        dbapi.update_container(self.context, container1.container_type,
                               container1.id,
                               {'name': new_name})
        self.assertRaises(exception.ContainerAlreadyExists,
                          dbapi.update_container, self.context,
                          container2.container_type,
                          container2.id, {'name': new_name})

    def test_update_container_not_found(self):
        container_uuid = uuidutils.generate_uuid()
        new_image = 'new-image'
        self.assertRaises(exception.ContainerNotFound,
                          dbapi.update_container, self.context,
                          consts.TYPE_CONTAINER,
                          container_uuid, {'image': new_image})

    def test_update_container_uuid(self):
        container = utils.create_test_container(context=self.context)
        self.assertRaises(exception.InvalidParameterValue,
                          dbapi.update_container, self.context,
                          container.container_type,
                          container.id, {'uuid': ''})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
The FilterScheduler is for scheduling container to a host according to
your filters configured.
You can customize this scheduler by specifying your own Host Filters.
"""

from oslo_log.log import logging

from zun.common import exception
from zun.common.i18n import _
import zun.conf
from zun import objects
from zun.scheduler.client import report
from zun.scheduler import driver
from zun.scheduler import filters
from zun.scheduler.host_state import HostState
from zun.scheduler import utils


CONF = zun.conf.CONF
LOG = logging.getLogger(__name__)


class FilterScheduler(driver.Scheduler):
    """Scheduler that can be used for filtering zun compute."""

    def __init__(self):
        super(FilterScheduler, self).__init__()
        self.filter_handler = filters.HostFilterHandler()
        filter_classes = self.filter_handler.get_matching_classes(
            CONF.scheduler.available_filters)
        self.filter_cls_map = {cls.__name__: cls for cls in filter_classes}
        self.filter_obj_map = {}
        self.enabled_filters = self._choose_host_filters(self._load_filters())
        self.placement_client = report.SchedulerReportClient()

    def _schedule(self, context, container, extra_specs, alloc_reqs_by_rp_uuid,
                  provider_summaries, allocation_request_version=None):
        """Picks a host according to filters."""
        elevated = context.elevated()

        # NOTE(jaypipes): provider_summaries being None is treated differently
        # from an empty dict. provider_summaries is None when we want to grab
        # all compute nodes.
        # The provider_summaries variable will be an empty dict when the
        # Placement API found no providers that match the requested
        # constraints, which in turn makes compute_uuids an empty list and
        # objects.ComputeNode.list will return an empty list
        # also, which will eventually result in a NoValidHost error.
        compute_uuids = None
        if provider_summaries is not None:
            compute_uuids = list(provider_summaries.keys())
        if compute_uuids is None:
            nodes = objects.ComputeNode.list(context)
        else:
            nodes = objects.ComputeNode.list(
                context, filters={'rp_uuid': compute_uuids})

        services = self._get_services_by_host(context)
        hosts = services.keys()
        nodes = [node for node in nodes if node.hostname in hosts]
        host_states = self.get_all_host_state(nodes, services)
        hosts = self.filter_handler.get_filtered_objects(self.enabled_filters,
                                                         host_states,
                                                         container,
                                                         extra_specs)
        if not hosts:
            msg = _("Is the appropriate service running?")
            raise exception.NoValidHost(reason=msg)

        # Attempt to claim the resources against one or more resource
        # providers, looping over the sorted list of possible hosts
        # looking for an allocation_request that contains that host's
        # resource provider UUID
        claimed_host = None
        for host in hosts:
            cn_uuid = host.uuid
            if cn_uuid not in alloc_reqs_by_rp_uuid:
                msg = ("A host state with uuid = '%s' that did not have a "
                       "matching allocation_request was encountered while "
                       "scheduling. This host was skipped.")
                LOG.debug(msg, cn_uuid)
                continue

            alloc_reqs = alloc_reqs_by_rp_uuid[cn_uuid]
            # TODO(jaypipes): Loop through all allocation_requests instead
            # of just trying the first one. For now, since we'll likely
            # want to order the allocation_requests in the future based on
            # information in the provider summaries, we'll just try to
            # claim resources using the first allocation_request
            alloc_req = alloc_reqs[0]
            if utils.claim_resources(
                    elevated, self.placement_client, container, alloc_req,
                    allocation_request_version=allocation_request_version):
                claimed_host = host
                break

        if claimed_host is None:
            # We weren't able to claim resources in the placement API
            # for any of the sorted hosts identified. So, clean up any
            # successfully-claimed resources for prior containers in
            # this request and return an empty list which will cause
            # select_destinations() to raise NoValidHost
            msg = _("Unable to successfully claim against any host.")
            raise exception.NoValidHost(reason=msg)

        # Now consume the resources so the filter/weights will change for
        # the next container.
        self._consume_selected_host(claimed_host, container)

        return claimed_host

    def select_destinations(self, context, containers, extra_specs,
                            alloc_reqs_by_rp_uuid, provider_summaries,
                            allocation_request_version=None):
        """Selects destinations by filters."""
        dests = []
        for container in containers:
            host = self._schedule(context, container, extra_specs,
                                  alloc_reqs_by_rp_uuid, provider_summaries,
                                  allocation_request_version)
            host_state = dict(host=host.hostname, nodename=None,
                              limits=host.limits)
            dests.append(host_state)

        if len(dests) < 1:
            reason = _('There are not enough hosts available.')
            raise exception.NoValidHost(reason=reason)

        return dests

    def _choose_host_filters(self, filter_cls_names):
        """Choose good filters

        Since the caller may specify which filters to use we need
        to have an authoritative list of what is permissible. This
        function checks the filter names against a predefined set
        of acceptable filters.
        """
        if not isinstance(filter_cls_names, (list, tuple)):
            filter_cls_names = [filter_cls_names]

        good_filters = []
        bad_filters = []
        for filter_name in filter_cls_names:
            if filter_name not in self.filter_obj_map:
                if filter_name not in self.filter_cls_map:
                    bad_filters.append(filter_name)
                    continue
                filter_cls = self.filter_cls_map[filter_name]
                self.filter_obj_map[filter_name] = filter_cls()
            good_filters.append(self.filter_obj_map[filter_name])
        if bad_filters:
            msg = ", ".join(bad_filters)
            raise exception.SchedulerHostFilterNotFound(filter_name=msg)
        return good_filters

    def _load_filters(self):
        return CONF.scheduler.enabled_filters

    def _get_services_by_host(self, context):
        """Get a dict of services indexed by hostname"""
        return {service.host: service
                for service in objects.ZunService.list_by_binary(
                    context,
                    'zun-compute')}

    def get_all_host_state(self, nodes, services):
        host_states = []
        for node in nodes:
            host_state = HostState(node.hostname)
            host_state.update(compute_node=node,
                              service=services.get(node.hostname))
            host_states.append(host_state)
        return host_states

    @staticmethod
    def _consume_selected_host(selected_host, container):
        LOG.debug("Selected host: %(host)s", {'host': selected_host})
        selected_host.consume_from_request(container)



# -*- encoding: utf-8 -*-
#
# Copyright © 2012 eNovance <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg


periodic_opts = [
    cfg.IntOpt('periodic_interval_max',
               default=60,
               help='Max interval size between periodic tasks execution in '
                    'seconds.'),
    cfg.IntOpt('service_down_time',
               default=180,
               help='Max interval size between periodic tasks execution in '
                    'seconds.'),
    cfg.IntOpt('sync_container_state_interval',
               default=60,
               help="""
Interval to sync container states between the database and the docker.

The interval that Zun checks the actual container state and
the state that Zun has recorded in its database. If they are inconsistent,
Zun will update the database according to the actual container state.

Possible values:
* 0: Will run at the default periodic interval.
* Any value < 0: Disables the option.
* Any positive integer in seconds.

"""),
]


ALL_OPTS = (periodic_opts)


def register_opts(conf):
    conf.register_opts(ALL_OPTS)


def list_opts():
    return {"DEFAULT": ALL_OPTS}



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from oslo_utils import uuidutils
from testtools.matchers import HasLength

from zun.common import consts
from zun import objects
from zun.tests.unit.db import base
from zun.tests.unit.db import utils


class TestContainerObject(base.DbTestCase):

    def setUp(self):
        super(TestContainerObject, self).setUp()
        self.fake_cpuset = utils.get_cpuset_dict()
        self.fake_container = utils.get_test_container(
            cpuset=self.fake_cpuset, cpu_policy='dedicated')
        self.fake_container.pop('capsule_id')

    def test_get_by_uuid(self):
        uuid = self.fake_container['uuid']
        container_type = self.fake_container['container_type']
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            container = objects.Container.get_by_uuid(self.context, uuid)
            mock_get_container.assert_called_once_with(
                self.context, container_type, uuid)
            self.assertEqual(self.context, container._context)

    def test_get_by_name(self):
        name = self.fake_container['name']
        container_type = self.fake_container['container_type']
        with mock.patch.object(self.dbapi, 'get_container_by_name',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            container = objects.Container.get_by_name(self.context, name)
            mock_get_container.assert_called_once_with(
                self.context, container_type, name)
            self.assertEqual(self.context, container._context)

    def test_get_container_any_type(self):
        mapping = {
            consts.TYPE_CONTAINER: objects.Container,
            consts.TYPE_CAPSULE: objects.Capsule,
            consts.TYPE_CAPSULE_CONTAINER: objects.CapsuleContainer,
            consts.TYPE_CAPSULE_INIT_CONTAINER: objects.CapsuleInitContainer,
        }
        for type, container_cls in mapping.items():
            self._test_get_container_any_type(type, container_cls)

    def _test_get_container_any_type(self, container_type, container_cls):
        fake_container = utils.get_test_container(
            container_type=container_type)
        uuid = fake_container['uuid']
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = fake_container
            container = objects.Container.get_container_any_type(
                self.context, uuid)
            mock_get_container.assert_called_once_with(
                self.context, consts.TYPE_ANY, uuid)
            self.assertEqual(self.context, container._context)
            self.assertIsInstance(container, container_cls)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'list_containers',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_container]
            containers = objects.Container.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(containers, HasLength(1))
            self.assertIsInstance(containers[0], objects.Container)
            self.assertEqual(self.context, containers[0]._context)

    def test_list_by_host(self):
        with mock.patch.object(self.dbapi, 'list_containers',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_container]
            containers = objects.Container.list_by_host(self.context,
                                                        'test_host')
            mock_get_list.assert_called_once_with(
                self.context, consts.TYPE_CONTAINER, {'host': 'test_host'},
                None, None, None, None)
            self.assertThat(containers, HasLength(1))
            self.assertIsInstance(containers[0], objects.Container)
            self.assertEqual(self.context, containers[0]._context)

    def test_list_with_filters(self):
        with mock.patch.object(self.dbapi, 'list_containers',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_container]
            filt = {'status': 'Running'}
            containers = objects.Container.list(self.context,
                                                filters=filt)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(containers, HasLength(1))
            self.assertIsInstance(containers[0], objects.Container)
            self.assertEqual(self.context, containers[0]._context)
            mock_get_list.assert_called_once_with(self.context,
                                                  consts.TYPE_CONTAINER,
                                                  filters=filt,
                                                  limit=None, marker=None,
                                                  sort_key=None, sort_dir=None)

    def test_create(self):
        with mock.patch.object(self.dbapi, 'create_container',
                               autospec=True) as mock_create_container:
            mock_create_container.return_value = self.fake_container
            container_dict = dict(self.fake_container)
            container_dict['cpuset'] = objects.container.Cpuset._from_dict(
                container_dict['cpuset'])
            container = objects.Container(self.context, **container_dict)
            container.create(self.context)
            mock_create_container.assert_called_once_with(self.context,
                                                          self.fake_container)
            self.assertEqual(self.context, container._context)

    def test_status_reason_in_fields(self):
        with mock.patch.object(self.dbapi, 'create_container',
                               autospec=True) as mock_create_container:
            mock_create_container.return_value = self.fake_container
            container_dict = dict(self.fake_container)
            container_dict['cpuset'] = objects.container.Cpuset._from_dict(
                container_dict['cpuset'])
            container = objects.Container(self.context, **container_dict)
            self.assertTrue(hasattr(container, 'status_reason'))
            container.status_reason = "Docker Error happened"
            container.create(self.context)
            self.assertEqual(
                "Docker Error happened",
                mock_create_container.call_args_list[0][0][1]['status_reason'])

    def test_destroy(self):
        uuid = self.fake_container['uuid']
        container_type = self.fake_container['container_type']
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            with mock.patch.object(self.dbapi, 'destroy_container',
                                   autospec=True) as mock_destroy_container:
                container = objects.Container.get_by_uuid(self.context, uuid)
                container.destroy()
                mock_get_container.assert_called_once_with(
                    self.context, container_type, uuid)
                mock_destroy_container.assert_called_once_with(
                    None, container_type, uuid)
                self.assertEqual(self.context, container._context)

    def test_save(self):
        uuid = self.fake_container['uuid']
        container_type = self.fake_container['container_type']
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            with mock.patch.object(self.dbapi, 'update_container',
                                   autospec=True) as mock_update_container:
                container = objects.Container.get_by_uuid(self.context, uuid)
                container.image = 'container.img'
                container.memory = '512m'
                container.environment = {"key1": "val", "key2": "val2"}
                container.save()

                mock_get_container.assert_called_once_with(
                    self.context, container_type, uuid)
                mock_update_container.assert_called_once_with(
                    None, container_type, uuid,
                    {'image': 'container.img',
                     'environment': {"key1": "val", "key2": "val2"},
                     'memory': '512m'})
                self.assertEqual(self.context, container._context)

    def test_refresh(self):
        uuid = self.fake_container['uuid']
        container_type = self.fake_container['container_type']
        new_uuid = uuidutils.generate_uuid()
        returns = [dict(self.fake_container, uuid=uuid),
                   dict(self.fake_container, uuid=new_uuid)]
        expected = [mock.call(self.context, container_type, uuid),
                    mock.call(self.context, container_type, uuid)]
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               side_effect=returns,
                               autospec=True) as mock_get_container:
            container = objects.Container.get_by_uuid(self.context, uuid)
            self.assertEqual(uuid, container.uuid)
            container.refresh()
            self.assertEqual(new_uuid, container.uuid)
            self.assertEqual(expected, mock_get_container.call_args_list)
            self.assertEqual(self.context, container._context)

    @mock.patch('zun.objects.PciDevice.list_by_container_uuid')
    def test_obj_load_attr_pci_devices(self, mock_list):
        uuid = self.fake_container['uuid']
        dev_dict = {'dev_id': 'fake_dev_id',
                    'container_uuid': 'fake_container_uuid'}
        pci_devices = [objects.PciDevice.create(self.context, dev_dict)]
        mock_list.return_value = pci_devices
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            container = objects.Container.get_by_uuid(self.context, uuid)
            container.obj_load_attr('pci_devices')
            self.assertEqual(pci_devices, container.pci_devices)

    @mock.patch('zun.objects.ExecInstance.list_by_container_id')
    def test_obj_load_attr_exec_instances(self, mock_list):
        uuid = self.fake_container['uuid']
        exec_dict = {'exec_id': 'fake_exec_id',
                     'container_id': self.fake_container['id']}
        exec_inst = objects.ExecInstance(self.context, **exec_dict)
        exec_inst.create(self.context)
        exec_insts = [exec_inst]
        mock_list.return_value = exec_insts
        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                               autospec=True) as mock_get_container:
            mock_get_container.return_value = self.fake_container
            container = objects.Container.get_by_uuid(self.context, uuid)
            container.obj_load_attr('exec_instances')
            self.assertEqual(exec_insts, container.exec_instances)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from zun.common import exception
from zun.common import mount
from zun.tests import base


class TestMounter(base.BaseTestCase):
    def setUp(self):
        super(TestMounter, self).setUp()
        self.mountinfo = "/dev/0 /path/to/0 type0 flags 0 0\n" \
                         "/dev/1 /path/to/1 type1 flags 0 0\n" \
                         "/dev/2 /path/to/2 type2 flags,1,2=3 0 0\n"
        self.mounts = [
            mount.MountInfo(device='/dev/0', mountpoint='/path/to/0',
                            fstype='type0', opts='flags'),
            mount.MountInfo(device='/dev/1', mountpoint='/path/to/1',
                            fstype='type1', opts='flags'),
            mount.MountInfo(device='/dev/2', mountpoint='/path/to/2',
                            fstype='type2', opts='flags,1,2=3'),
        ]

    @mock.patch('zun.common.utils.execute')
    def test_mount(self, mock_execute):
        fake_devpath = '/dev/3'
        fake_mp = '/path/to/3'
        fake_fstype = 'ext4'
        mounter = mount.Mounter()
        mounter.mount(fake_devpath, fake_mp, fake_fstype)
        mock_execute.assert_called_once_with(
            'mount', '-t', fake_fstype, fake_devpath, fake_mp,
            run_as_root=True)

    @mock.patch('zun.common.utils.execute')
    def test_unmount_error(self, mock_execute):
        fake_devpath = '/dev/3'
        fake_mp = '/path/to/3'
        fake_fstype = 'ext4'
        mock_execute.side_effect = exception.CommandError()
        mounter = mount.Mounter()
        self.assertRaises(exception.MountException,
                          mounter.mount, fake_devpath, fake_mp, fake_fstype)

    @mock.patch('zun.common.utils.execute')
    def test_read_mounts(self, mock_execute):
        mock_execute.return_value = (self.mountinfo, '')
        expected_mounts = [str(m) for m in self.mounts]
        mounter = mount.Mounter()
        mounts = [str(m) for m in mounter.read_mounts()]
        self.assertEqual(len(expected_mounts), len(mounts))
        for m in mounts:
            self.assertIn(m, expected_mounts)

    @mock.patch('zun.common.utils.execute')
    def test_read_mounts_error(self, mock_execute):
        mock_execute.side_effect = exception.CommandError()
        mounter = mount.Mounter()
        self.assertRaises(exception.FileNotFound, mounter.read_mounts)

    @mock.patch('zun.common.utils.execute')
    @mock.patch('zun.common.mount.Mounter.read_mounts')
    def test_get_mps_by_device(self, mock_read_mounts, mock_execute):
        mock_execute.return_value = (self.mountinfo, '')
        mock_read_mounts.return_value = self.mounts
        mounter = mount.Mounter()
        self.assertEqual(['/path/to/0'],
                         mounter.get_mps_by_device('/dev/0'))
